from typing import *


def isshape(obj: Any, shape: Any) -> bool:
    """
    Check if an object matches a given generic type expression.

    :param obj: The object to be checked.
    :type obj: Any
    :param shape: The generic type expression to match against.
    :type shape: Any
    :return: True if the object matches the type expression, False otherwise.
    :rtype: bool
    """
    if isinstance(obj, Tuple) and isinstance(shape, Tuple):
        for _obj, _type in zip(obj, shape):
            if not isinstance(_obj, _type):
                return False

    origin = get_origin(shape)

    if origin == list:
        if not isinstance(obj, origin):
            return False
        element_type = get_args(shape)[0]
        return all(isshape(item, element_type) for item in obj)

    elif origin in (dict, map):
        if not isinstance(obj, origin):
            return False
        key_type, value_type = get_args(shape)
        return all(isshape(key, key_type) and isshape(value, value_type) for key, value in obj.items())

    elif origin == Union:
        return any(isshape(obj, t) for t in get_args(shape))

    elif origin == None:
        return isinstance(obj, shape)

    else:
        if not isinstance(obj, origin):
            return False
        element_type = get_args(shape)[0]
        return all(isshape(item, element_type) for item in obj)



import time
import random


def to_base64(num: int) -> str:
    BASE64_CHARS = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_"
    if num == 0:
        return BASE64_CHARS[0]

    result = []
    while num:
        remainder = num & 0x3F  # 等价于 num % 64
        result.append(BASE64_CHARS[remainder])
        num >>= 6
    return ''.join(reversed(result))

def reverse_bits(n: int) -> int:
    res = 0
    for i in range(64):
        res = (res << 1) | ((n >> i) & 1)
    return res

def toid(msid:int = 0) -> str:
    # 获取纳秒级时间戳
    nanoseconds_timestamp: int = time.time_ns()
    rev_ts: int = reverse_bits(nanoseconds_timestamp)
    return to_base64(rev_ts + msid)

print(toid())



from typing import Optional


class SimpleVersion(str):
    def __init__(self, version):
        self.version: str = version
        self.version_resolved: tuple = self.resolver()
        self.version_number: Optional[int] = None

    def resolver(self) -> tuple:
        # 将版本字符串转换为元组
        return tuple(map(int, (self.version.split("."))))

    # 版本大小比较的运算符重载方法
    # 大于
    def __gt__(self, other: 'SimpleVersion'):
        # 类型检查
        if not isinstance(other, SimpleVersion):
            raise TypeError("比较对象不是SimpleVersion类型")
        if self.version_number:
            return self.version_number > other.version_number
        else:
            # 按位比较 self.version_resolved 和 other.version_resolved
            for i in range(len(self.version_resolved)):
                if self.version_resolved[i] > other.version_resolved[i]:
                    return True
                elif self.version_resolved[i] < other.version_resolved[i]:
                    return False
            return False

    # 小于
    def __lt__(self, other: 'SimpleVersion'):
        if not isinstance(other, SimpleVersion):
            raise TypeError("比较对象不是SimpleVersion类型")
        if self.version_number:
            return self.version_number < other.version_number
        else:
            for i in range(len(self.version_resolved)):
                if self.version_resolved[i] < other.version_resolved[i]:
                    return True
                elif self.version_resolved[i] > other.version_resolved[i]:
                    return False
            return False



from typing import Union


class Version:
    def __init__(self, version_name: str = None, version_code: int = None, life_cycle: str = 'release'):
        self.name = version_name
        self.format_name: tuple = None
        self.code = version_code
        self.life_cycle = life_cycle.lower()

    def format(self):
        if self.name and self.name.replace('.', '').isdigit():
            self.format_name = tuple(self.name.split('.'))

    def __str__(self):
        return self.name

    def __repr__(self):
        return f'<Version {self.life_cycle}-{self.name} {self.code}>'

    def __eq__(self, other: Union[str, 'Version']):
        if isinstance(other, str):
            other = Version(other)
        if isinstance(other, Version):
            if self.code and other.code:
                return self.code == other.code
            else:
                return self.name == other.name
        else:
            raise TypeError(f'Cannot compare {type(other)} with {type(self)}')

    def __lt__(self, other: Union[str, 'Version']):
        if isinstance(other, str):
            other = Version(other)
            if self.code and other.code:
                return self.code < other.code
